@@ -1,6 +1,6 @@
 use super::Handler;
-use crate::body::BoxBody;
-use http::{Request, Response};
+use crate::response::Response;
+use http::Request;
 use std::{
     convert::Infallible,
     fmt,
@@ -12,9 +12,9 @@ use tower_service::Service;
 /// An adapter that makes a [`Handler`] into a [`Service`].
 ///
 /// Created with [`Handler::into_service`].
-pub struct IntoService<H, B, T> {
+pub struct IntoService<H, T, B> {
     handler: H,
-    _marker: PhantomData<fn() -> (B, T)>,
+    _marker: PhantomData<fn() -> (T, B)>,
 }
 
 #[test]
@@ -24,7 +24,7 @@ fn traits() {
     assert_sync::<IntoService<(), NotSendSync, NotSendSync>>();
 }
 
-impl<H, B, T> IntoService<H, B, T> {
+impl<H, T, B> IntoService<H, T, B> {
     pub(super) fn new(handler: H) -> Self {
         Self {
             handler,
@@ -33,7 +33,7 @@ impl<H, B, T> IntoService<H, B, T> {
     }
 }
 
-impl<H, B, T> fmt::Debug for IntoService<H, B, T> {
+impl<H, T, B> fmt::Debug for IntoService<H, T, B> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_tuple("IntoService")
             .field(&format_args!("..."))
@@ -41,7 +41,7 @@ impl<H, B, T> fmt::Debug for IntoService<H, B, T> {
     }
 }
 
-impl<H, B, T> Clone for IntoService<H, B, T>
+impl<H, T, B> Clone for IntoService<H, T, B>
 where
     H: Clone,
 {
@@ -53,12 +53,12 @@ where
     }
 }
 
-impl<H, T, B> Service<Request<B>> for IntoService<H, B, T>
+impl<H, T, B> Service<Request<B>> for IntoService<H, T, B>
 where
-    H: Handler<B, T> + Clone + Send + 'static,
+    H: Handler<T, B> + Clone + Send + 'static,
     B: Send + 'static,
 {
-    type Response = Response<BoxBody>;
+    type Response = Response;
     type Error = Infallible;
     type Future = super::future::IntoServiceFuture;
 
