@@ -60,7 +60,7 @@ macro_rules! define_rejection {
         #[allow(deprecated)]
         impl $crate::response::IntoResponse for $name {
             fn into_response(self) -> $crate::response::Response {
-                let mut res = http::Response::new($crate::body::boxed(http_body::Full::from($body)));
+                let mut res = http::Response::new($crate::body::boxed(crate::body::Full::from($body)));
                 *res.status_mut() = http::StatusCode::$status;
                 res
             }
@@ -103,7 +103,7 @@ macro_rules! define_rejection {
         impl IntoResponse for $name {
             fn into_response(self) -> $crate::response::Response {
                 let mut res =
-                    http::Response::new($crate::body::boxed(http_body::Full::from(format!(concat!($body, ": {}"), self.0))));
+                    http::Response::new($crate::body::boxed(crate::body::Full::from(format!(concat!($body, ": {}"), self.0))));
                 *res.status_mut() = http::StatusCode::$status;
                 res
             }
