@@ -65,7 +65,6 @@ impl RouteId {
 }
 
 /// The router type for composing handlers and services.
-#[derive(Debug)]
 pub struct Router<B = Body> {
     routes: HashMap<RouteId, Endpoint<B>>,
     node: Node,
@@ -93,6 +92,17 @@ where
     }
 }
 
+impl<B> fmt::Debug for Router<B> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Router")
+            .field("routes", &self.routes)
+            .field("node", &self.node)
+            .field("fallback", &self.fallback)
+            .field("nested_at_root", &self.nested_at_root)
+            .finish()
+    }
+}
+
 pub(crate) const NEST_TAIL_PARAM: &str = "axum_nest";
 const NEST_TAIL_PARAM_CAPTURE: &str = "/*axum_nest";
 
